/* Copyright (C) 2009, Chuck Shotton, http://www.shotton.com/
 * Licensed under the Creative Commons Attribution-Share Alike 3.0 license.
 * See http://creativecommons.org/licenses/by-sa/3.0/ for details.
 */

package com.concluent.rssNimbus;

import org.restlet.data.*;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Post;
import org.restlet.resource.ServerResource;

import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Handler for /ping URLs that update publications and triggers subscriber notifications.
 * @author Chuck Shotton
 *
 */
public class PingHandler extends ServerResource{

	static final boolean USE_TASK_QUEUE = true;
	public static final int CONNECT_TIMEOUT = 2000;
	public static final int READ_TIMEOUT = 10000;
	
	@Post
	public Representation acceptRepresentation(Representation entity) {
	//Could process some post requests here and possibly write to the datastore
		String res = "\n";
		Form form = new Form (entity);
		String url = form.getFirstValue ("url");
		
		setStatus (Status.SUCCESS_OK);
		res += "Ping: checking status of " + url;

		//fetch the URL contents
		String rss = "";
		String line = "";
		boolean pingOK = true;
		
		try {
			URL u = new URL (url);
			URLConnection uc = u.openConnection ();
			HttpURLConnection huc = (HttpURLConnection) uc;
		
			huc.setConnectTimeout(CONNECT_TIMEOUT);
			huc.setReadTimeout(READ_TIMEOUT);
			huc.setRequestMethod("GET");
			huc.setUseCaches(false);
			
			BufferedReader in = new BufferedReader (new InputStreamReader (uc.getInputStream ()));
			int rcode = huc.getResponseCode ();
			String response = huc.getResponseMessage ();
			
			while ((line = in.readLine()) != null) {
				rss += line + "\n";
			}
//System.out.println ("GET response was: " + rcode + ", " + res);			
			in.close ();

		} catch (Exception e) {
			pingOK = false;
			res += "\nError trying to access document.";
			e.printStackTrace();
		}
		
		if (pingOK) {			
			//compute the checksum
			String checksum = Utility.makeMD5(rss);

			res += "\nChecksum for publication is " + checksum;
			
			//get the current publication record
			PublicationObject op = PublicationObject.getPublicationByURL(url);
			if (op == null) {
				//or make a new one
				op = new PublicationObject (url, checksum); //use the actual checksum so we don't waste time below trying to notify nonexistent subscribers
				System.out.println("Making new publication object for " + url);
				res += "\nThis publication has not been seen before.";
			}
			
			if (op != null) {
				op.makeTimestamp(); //track that we've heard from the publisher now
				
				//if the checksums are different
				if (!checksum.equalsIgnoreCase(op.getHash())) {
					//save the new hash
					op.setHash(checksum);
					
					//notify subscribers
					int count = notifySubscribers (op, USE_TASK_QUEUE);
					res += "\nThis publication has been modified. " + count + " subscribers notified.";
				}
				else {
					res += "\nThis publication has not changed.";
				}
				res += "\nTimestamp: " + op.getTimestamp();
				op.savePublication();
			}
			else {
				System.out.println("ERROR!! Had a problem creating a publication object for " + url);
			}
		}
		System.out.println (res);
		return new StringRepresentation (res);
 	}

	/**
	 * 
	 * @param p PublicationObject that has been updated
	 * @param queued boolean indicating whether to perform the notifications as individual GAE tasks (true) or synchronously (false)
	 * @return number of subscribers that will be notified
	 */
	private int notifySubscribers (PublicationObject p, boolean queued) {
		// TODO add code to notify subscribers of new publication
		int count = Utility.notifyAllSubscribers (p.getURL(), queued);
		return count;
	}
		
}