/* Copyright (C) 2009, Chuck Shotton, http://www.shotton.com/
 * Licensed under the Creative Commons Attribution-Share Alike 3.0 license.
 * See http://creativecommons.org/licenses/by-sa/3.0/ for details.
 */

package com.concluent.rssNimbus;
import org.restlet.data.*;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.Post;
import org.restlet.resource.ServerResource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Handler for /pleaseNotify URLs received from subscribers via HTTP POST
 * @author Chuck Shotton
 *
 */
public class PleaseNotifyHandler extends ServerResource{

	@Post
	public Representation acceptRepresentation(Representation entity) {
		Representation result = null;
		setStatus (Status.SERVER_ERROR_NOT_IMPLEMENTED);
		boolean successFlag = true;
		boolean challengeNeeded = false;
		String theMsg = "";
		
		//extract the arguments
		Form form = new Form (entity);
		String url = form.getFirstValue ("url1");
		String notifyProcedure = form.getFirstValue ("notifyProcedure");
		String port = form.getFirstValue ("port");
		String path = form.getFirstValue ("path");
		String protocol = form.getFirstValue ("protocol");
		String domain = form.getFirstValue ("domain");
		String challenge = form.getFirstValue ("challenge");	//this is for the extended challenge/response code
		String address = "";
		
		//check the arguments
		if (url == null) {
			successFlag = false;
			theMsg += "\nERROR: No URL supplied for notification request.";
		}
		if (notifyProcedure == null) {
			successFlag = false;
			theMsg += "\nERROR: No notification procedure supplied for notification request.";
		}
		if (port == null) {
			successFlag = false;
			theMsg += "\nERROR: No port supplied for notification request.";
		} 
		else if (!port.equals("80") && !port.equals("443")) {
			successFlag = false;
			theMsg += "\nERROR: rssNimbus only provides notifications to ports 80 and 443 due to GAE restrictions. Your port=" + port;
		}
		if (path == null) {
			successFlag = false;
			theMsg += "\nERROR: No path supplied for notification request.";
		}
		if (protocol == null) {
			successFlag = false;
			theMsg += "\nERROR: No protocol supplied for notification request.";
		}
		else if (!protocol.equalsIgnoreCase("http-post")) {
			successFlag = false;
			theMsg += "\nERROR: rssNimbus only supports notifications via http-post. Your protocol=" + protocol;
		}
		if (domain == null || domain.length()<3) {
			//we have to use the client's address instead of the domain name
			Request req = this.getRequest();
			address = req.getClientInfo().getAddress();
		}
		else {
			address = domain;
			challengeNeeded = true;
		}
		if (challenge==null || challenge.length()<1) {
			challenge = "rssNimbus";
		}

		//for each url
		int urlNum = 1;
		
		while (url != null && successFlag) {
		
			//get the publication record
			PublicationObject p = PublicationObject.getPublicationByURL(url);
			
			//if we have one
			if (p!=null) {
				
				//hit the notification handler to see if it's real and to get the client to load the current document
				SubscriptionObject sub = new SubscriptionObject (protocol, address, port, notifyProcedure, path, url);
				boolean notified=false;
				if (challengeNeeded) {
					notified = Utility.challengeOneSubscriber (sub, challenge);
				}
				else {
					notified = Utility.notifyOneSubscriber(sub);
				}
				
				if (notified) {
					// create a subscription record
					sub.saveSubscription();
				}
				else {
					//the handler wasn't there, so we're ignoring this request.
					successFlag = false;
					theMsg += "\nERROR: Notification handler validation attempt failed using:" + "\naddress: " + address + 
								"\nport=" + port + "\nprotocol=" + protocol + "\npath=" + path + "\nnotifyProcedure=" + notifyProcedure;
				}
			
				if (successFlag) {
					theMsg += "\nNotification request " + urlNum + " accepted:\nurl" + urlNum + "=" + url + 
								"\naddress: " + address + "\nnotifyProcedure: " + notifyProcedure
							+ "\nport: " + port + "\npath: " + path + "\nprotocol: " + protocol;
				}
				
				//get the next URL to check
				url = form.getFirstValue("url" + ++urlNum);
			}
			else {
				successFlag = false;
				theMsg += "\nERROR: Notification request " + urlNum + " rejected. Requested publication is not known to this server. url" + urlNum + "=" + url;
			}
		} //while
		
		//create the xml response
		
		try {
			//make a new dom tree
			DomRepresentation domRep = new DomRepresentation(MediaType.TEXT_XML);
			Document d = domRep.getDocument ();
			
			//make the notifyResult entity
			Element nr = d.createElement ("notifyResult");
			d.appendChild(nr);
			
			//make the success value
			Element success = d.createElement ("success");
			success.appendChild(d.createTextNode(Boolean.toString(successFlag)));
			nr.appendChild (success);
			
			//make the msg value
			Element msg = d.createElement ("msg");
			msg.appendChild(d.createTextNode(theMsg));
			nr.appendChild (msg);
			
			//finish the xml document
			d.normalizeDocument ();
			result = domRep;
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return result;
 	}

}