/* Copyright (C) 2009, Chuck Shotton, http://www.shotton.com/
 * Licensed under the Creative Commons Attribution-Share Alike 3.0 license.
 * See http://creativecommons.org/licenses/by-sa/3.0/ for details.
 */

package com.concluent.rssNimbus;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdentityType;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

/**
 * Defines a JDO object for Publications
 * @author Chuck Shotton
 *
 */
@PersistenceCapable(identityType = IdentityType.APPLICATION, detachable="true")
public class PublicationObject {
	@PrimaryKey
	@Persistent
	private String url="";
	@Persistent
	private long timestamp=0; //java.lang.System.currentTimeMillis() 
	@Persistent
	private String hash="";

	/**
	 * read a PublicationObject from the GAE persistent storage and return a new object with its values
	 * @param url - the key of the publication to retrieve
	 * @return PublicationObject matching the requested url key
	 */
	public static PublicationObject getPublicationByURL (String url) {
		PublicationObject opub = null;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		try {
			opub = pm.getObjectById(PublicationObject.class, url);
			opub = pm.detachCopy (opub);
		} catch (Exception e) {
			//e.printStackTrace();
		} finally {
			pm.close();
		}
		return opub;
	}
	
	/**
	 * 
	 * @param p PublicationObject to delete from the object store
	 * @return boolean indicating success
	 */
	public static boolean deletePublication (PublicationObject p) {
		boolean success = true;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		try {
			PublicationObject opub = pm.getObjectById(PublicationObject.class, p.getURL());
			pm.deletePersistent(opub);
		} catch (Exception e) {
			e.printStackTrace();
			success = false;
		} finally {
			pm.close();
		}
		return success;
	}
	
	/**
	 * You probably don't want to use this constructor....
	 */
	public PublicationObject () {
	}
	
	/**
	 * create a new PublicationObject and initialize it with a current timestamp
	 * @param url String representation of the source URL for the publication
	 * @param hash String representation of the MD5 hash for the publication's content
	 */
	public PublicationObject (String url, String hash) {
		this.url = url;
		this.hash = hash;
		makeTimestamp ();
	}
	
	/**
	 * save the publication object's contents to the GAE persistent data store
	 * @return true if successful, false if it failed.
	 */
	public boolean savePublication () {
		boolean success = true;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		
		try {
			pm.makePersistent(this);
		} catch (Exception e) {
			e.printStackTrace ();
			success = false;
		} finally {
			pm.close ();
		}
		return success;
	}
	
	public String getURL () {
		return this.url;
	}
	
	public void setURL (String url) {
		this.url = url;
	}
	
	public String getHash () {
		return this.hash;
	}
	
	public void setHash (String hash) {
		this.hash = hash;
	}
	
	public long getTimestamp () {
		return this.timestamp;
	}
	
	/**
	 * set the timestamp to the current time in milliseconds since epoch
	 */
	public void makeTimestamp () {
		this.timestamp = java.lang.System.currentTimeMillis();
	}
	
	public void setTimestamp (long timestamp) {
		this.timestamp = timestamp;
	}
}
